-- Setting up libs
iFriends = LibStub("AceAddon-3.0"):NewAddon("iFriends", "AceEvent-3.0", "AceTimer-3.0")

local addon = iFriends
local broker = LibStub("LibDataBroker-1.1")
local feed = broker:NewDataObject("iFriends", {
	type = "data source",
	text = "Loading...",
	icon = "Interface\\Addons\\iFriends\\Fubar_FriendsFu",

	OnClick = function()
		ToggleFriendsFrame(1) -- if we click, the friends frame shall be toggled
	end,

	OnEnter = function(anchor)
		addon:ShowTooltip(anchor) -- if we hover with the mouse, the tooltip is displayed
	end,
})
local qtip = LibStub("LibQTip-1.0")

local T = LibStub("LibTourist-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("iFriends")

local tip, flist, nflist -- the tip, friends list table and friends list table count
local NULL = "" --this string is displayed whenever a cell got no content, f.e. Race with local friends

------------------------------------------
-- Colors
------------------------------------------

local ColG = "|cfffed100" -- golden color
local ColBN = ("|cff%02x%02x%02x"):format(0.1*255, 0.8*255, 1*255) -- battle.net blue

local classColors = {} -- table of class colors: classColors[Priest] = white
-- thanks to Broker_Friends, saved me some minutes
for key, localized in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local color = RAID_CLASS_COLORS[key]
	classColors[localized] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end
for key, localized in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local color = RAID_CLASS_COLORS[key]
	classColors[localized] = ("|cff%02x%02x%02x"):format(color.r * 255, color.g * 255, color.b * 255)
end

------------------------------------------
-- OnEnable
------------------------------------------

function addon:OnEnable()
	ShowFriends() -- call the game to update the friends...
	self:ScheduleRepeatingTimer(ShowFriends, 55) -- ...every 55 seconds
	
	self:RegisterEvent("FRIENDLIST_UPDATE", "GotEvent")
	self:RegisterEvent("BN_FRIEND_INFO_CHANGED", "GotEvent")
	self:RegisterEvent("BN_FRIEND_LIST_SIZE_CHANGED", "GotEvent")
end

------------------------------------------
-- GotEvent
------------------------------------------

function addon:GotEvent()
	local nFriends, nFriendsOn = GetNumFriends()
	
	local nBNFriends, nBNFriendsOn = 0, 0
	if BNFeaturesEnabledAndConnected() then
		nBNFriends, nBNFriendsOn = BNGetNumFriends()
	end
	
	local nTotal = nFriends + nBNFriends
	local nTotalOn = nFriendsOn + nBNFriendsOn
	feed.text = ("%d/%d"):format(nTotalOn, nTotal) -- displays the "2/9" friends online count
	
	self:SetupFriendsData(nFriends, nFriendsOn, nBNFriends, nBNFriendsOn, nTotal, nTotalOn)
	
	if qtip:IsAcquired(self) then
		self:UpdateTooltip()
	end
end

------------------------------------------
-- SetupFriendsData
------------------------------------------

function addon:SetupFriendsData(nFriends, nFriendsOn, nBNFriends, nBNFriendsOn, nTotal, nTotalOn)
	local list = {{}}
	local iter = 0
	
	-- in this function we bring the friends data into a table.
	-- this enabled the ability to sort the data, on a later day
	for i = 1, nFriendsOn do
		local charName, level, class, area, isOnline, status, note = GetFriendInfo(i)
		
		if isOnline then
			iter = iter + 1
			list[iter] = {
				bn = false,
				wow = true,
				charname = charName,
				level = level,
				class = class,
				area = area,
				status = status,
				note = note,
			}
		end
	end
	
	if BNFeaturesEnabledAndConnected() then
		for i = 1, nBNFriendsOn do
			local pID, givenName, surname, _, _, _, isOnline, _, isAFK, isDND, broadcast, note, _, _ = BNGetFriendInfo(i)
			
			if isOnline then
				_, charName, client, realm, _, faction, race, class, _, area, level, _ = BNGetToonInfo(pID)
				
				-- check if our friend is playing WoW
				local isWOW = false
				if client == BNET_CLIENT_WOW then
					isWOW = true
				end
				
				-- prepare the real id name
				local realid = BATTLENET_NAME_FORMAT:format(givenName, surname)
				
				-- prepare the status text, f.e. <AFK>
				local status = ""
				if isAFK then
					status = ("<%s>"):format(AFK)
				elseif isDND then
					status = ("<%s>"):format(DND)
				end
				
				-- friends broadcasts override our "local" note
				-- we use the broadcast variable as a bool after that if, to encolor it blue in the tip if its a broadcast
				if broadcast == "" then
					broadcast = false
				else
					note = broadcast
					broadcast = true
				end
				
				iter = iter + 1
				list[iter] = {
					bn = true,
					wow = isWOW,
					pid = pID,
					client = client,
					realid = realid,
					realm = realm,
					charname = charName,
					level = level,
					class = class,
					faction = faction, -- 0 = horde, 1 = alliance
					race = race,
					area = area,
					status = status,
					note = note,
					broadcast = broadcast
				}
			end
		end
	end
	
	flist = list
	nflist = iter
end

------------------------------------------
-- OnClick callbacks
------------------------------------------

local function cb_FriendClick(_, name)
	SetItemRef(("player:%s"):format(name), ("|Hplayer:%s|h[%s]|h"):format(name, name), "LeftButton")
end

local function cb_BNFriendClick(_, f)
	local itemRef = ("%s:%d"):format(f.realid, f.pid)
	SetItemRef(("BNplayer:%s"):format(itemRef), ("|HBNplayer:%s|h[%s]|h"):format(itemRef, f.realid), "LeftButton")
end

------------------------------------------
-- UpdateTooltip
------------------------------------------

function addon:UpdateTooltip()
	tip:Clear() -- clear the tooltip for usage now
	if nflist == 0 then
		tip:SetCell(tip:AddHeader(), 1, L["You have no online friends."], nil, 7) -- this should never happen !! :P
		return
	end
		
	tip:AddHeader(
		("%s%s|r"):format(ColG, L["RealID"]),
		("%s%s|r"):format(ColG, L["Game/Realm"]),
		("%s%s|r"):format(ColG, L["Name"]),
		("%s%s|r"):format(ColG, L["Race"]),
		("%s%s|r"):format(ColG, L["Level"]),
		("%s%s|r"):format(ColG, L["Zone"]),
		("%s%s|r"):format(ColG, L["Note"])
	)
	
	for i = 1, nflist do
		local f = flist[i]
		local line
		
		if f.wow then
			local areaR, areaG, areaB = T:GetFactionColor(f.area)
			local level = GetQuestDifficultyColor(f.level)
			
			if f.bn then
				-- I set up this vars to get the color based on my faction. If I am Horde, this function displays my Horde friends green
				-- ...and my ally friends red. I just picked localized city names and do a check with em.
				local factioncity = {L["Hordecity"], L["Allycity"]}
				local factionR, factionG, factionB = T:GetFactionColor(factioncity[f.faction +1])
				
				if f.broadcast then
					f.note = ("%s%s|r"):format(ColBN, f.note)
				end
			
				line = tip:AddLine(
					("%s%s|r"):format(ColBN, f.realid),
					("|cff%02x%02x%02x%s|r"):format(factionR*255, factionG*255, factionB*255, f.realm),
					("%s%s%s|r"):format(classColors[f.class], f.status, f.charname),
					("|cff%02x%02x%02x%s|r"):format(factionR*255, factionG*255, factionB*255, f.race),
					("|cff%02x%02x%02x%s|r"):format(level.r*255, level.g*255, level.b*255, f.level),
					("|cff%02x%02x%02x%s|r"):format(areaR*255, areaG*255, areaB*255, f.area),
					("%s%s|r"):format(ColG, f.note or NULL)
				)
				tip:SetLineScript(line, "OnMouseDown", cb_BNFriendClick, f)
			else
				line = tip:AddLine(
					("%s%s|r"):format(ColG, NULL),
					("%s%s|r"):format(FRIENDS_OTHER_NAME_COLOR_CODE, L["local friend"]),
					("%s%s%s|r"):format(classColors[f.class], f.status, f.charname),
					("%s%s|r"):format(ColG, NULL),
					("|cff%02x%02x%02x%s|r"):format(level.r*255, level.g*255, level.b*255, f.level),
					("|cff%02x%02x%02x%s|r"):format(areaR*255, areaG*255, areaB*255, f.area),
					("%s%s|r"):format(ColG, f.note or NULL)
				)
				tip:SetLineScript(line, "OnMouseDown", cb_FriendClick, f.charname)
			end
		else
			local r, g, b, a = 0, 0, 0, 0
			if f.client == "SC2" then
				f.client, r, g, b, a = "Starcraft2", 0.1, 0.8, 1, 0.6
			elseif f.client == "D3" then
				f.client, r, g, b, a = "Diablo3", 1, 0.1, 0.1, 0.6
			end
			
			line = tip:AddLine(
				("%s%s|r"):format(ColBN, f.realid),
				f.client,
				NULL,
				NULL,
				NULL,
				NULL,
				("%s%s|r"):format(ColBN, f.note or NULL)
			)
			tip:SetLineColor(line, r, g, b, a)
		end
	end
end

------------------------------------------
-- ShowTooltip
------------------------------------------

function addon:ShowTooltip(anchor)
	ShowFriends()
	tip = qtip:Acquire(iFriends, 7, "LEFT", "CENTER", "CENTER", "CENTER", "CENTER", "CENTER", "LEFT")
	tip:SetAutoHideDelay(0.1, anchor)
	tip:SmartAnchorTo(anchor)
	self:UpdateTooltip()
	tip:Show()
end